/*************************************************
* daemonize.c
* Copyright (c) Rennie deGraaf, 2001-2005.  All rights reserved.
* $Id: daemonize.c 14 2005-07-26 02:00:59Z degraaf $
*
* Functions to create daemon processes
* daemonize() - Detach a process from the terminal and run it in the 
*  background
* is_daemon() - Detect if the current process is a daemon
*
* Note: daemonize() closes the logmsg facility, if it is open
*
* This file is part of the libwheel project.
*
* libwheel is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* libwheel is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with libwheel; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
**************************************************/

#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include "logmsg.h"


/*****************************
int is_daemon()
Detects if a process is a daemon
RETURNS: non-zero if the process is not a daemon
         zero otherwise
Note: - I assume that a process is a daemon if it's parent has a PID of 1.  
This may not be 100% accurate.
*/
int is_daemon()
{
    if (getppid() == 1)
        return 1;
    return 0;
}

/*****************************
void daemonize(char*, char*, char*)
Put the process in the background and make it act like a deamon
PARAMS:  const char* infile - name of file to use as stdin
         const char* outfile - name of file to use as stdout
         const char* errfile - name of file to use as stderr
Note: - aborts on failure, sends error messages through the logmsg facility.
      - logmsg will have to be re-opened after calling this function.
*/
void daemonize(const char* infile, const char* outfile, const char* errfile)
{
    int max_fd;
    int fd;
    
    /* return control to the command line by killing original process */
    switch(fork())
    {
        case -1:
            LOGMSG_LIB(fork);
            LOGMSG_FATAL_EXIT();
            abort();
            break;
        case 0: /* child */
            break;
        default: /* parent */
            exit(0);
    }
    
    /* now executing child process */
    /* make this process a process group leader */
    if(setsid() == -1)
    {
        LOGMSG_LIB(setsid);
        LOGMSG_FATAL_EXIT();
        abort();
    }
    
    /* kill group leader to make sure terminal cannot be recovered */
    switch(fork())
    {
        case -1:
            LOGMSG_LIB(fork);
            LOGMSG_FATAL_EXIT();
            abort();
        case 0:
            break;
        default:
            exit(0);
    }
    
    /* make sure no directory is in use (will have to use absolute paths from here on)*/
    if(chdir("/") == -1)
    {
        LOGMSG_LIB(chdir);
        LOGMSG_FATAL_EXIT();
        abort();
    }
    
    /* shut down logmsg */
    logmsg_close();
    
    /* close all open file descriptors */
    max_fd = getdtablesize();
    for (fd=0; fd<max_fd; fd++)
    {
        close(fd); /* don't want to error check because not all fds are in use */
    }

    /* open new stdin, stdout, stderr */
    /* if something screws up here, no error can be printed */
    if (infile != NULL) /* stdin */
    {
        if (open(infile, O_RDONLY|O_NOCTTY) != 0)
            abort();
    }
    if (outfile != NULL) /* stdout */
    {
        if (open(outfile, O_WRONLY|O_APPEND|O_NOCTTY|O_CREAT, 0600) != 1)
            abort();
    }
    if (errfile != NULL) /* stderr */
    {
        if (open(errfile, O_WRONLY|O_CREAT|O_NOCTTY|O_APPEND, 0600) != 2)
            abort();
    }
        
    return;
}

